local utils = require("utils")

local mod = {}

local FLAP_SOUND = sound.load("flappy_flap.wav")

local BACKGROUND_TEXTURE = image.load("flappy.png")
local FLAPPY_TEXTURE = image.load("flappy_skull.png")
image.center(FLAPPY_TEXTURE)
local PILLAR_TEXTURE_TOP = image.load("flappy_pillar.png")
local PILLAR_TEXTURE_BOTTOM = image.copy(PILLAR_TEXTURE_TOP)
image.fliph(PILLAR_TEXTURE_BOTTOM)
image.flipv(PILLAR_TEXTURE_BOTTOM)

local FLAPPY_X = 35
local FLAPPY_SIZE = 20
local FLAPPY_COLOR = color.new(255, 255, 0)

local FLAPPY_SPEED_GRAVITY = 0.13
local FLAPPY_SPEED_FLAPPING = 3.8

local PLAYFIELD_HEIGHT = 272
local PLAYFIELD_WIDTH = 200

local DRAW_X_OFFSET = 480 / 2 - PLAYFIELD_WIDTH / 2
local DRAW_Y_OFFSET = 272 / 2 - PLAYFIELD_HEIGHT / 2

local BAR_WIDTH = 22
local BAR_SPACING = 81
local BAR_COLOR = color.new(0, 200, 0)
local BAR_SPEED = 1.5

local FLAPPY_LAP_SCORE = 10

function mod.reset(STATE)
    STATE.flappy_y = PLAYFIELD_HEIGHT / 2
    STATE.flappy_speed = 0
    STATE.flappy_bar_x = PLAYFIELD_WIDTH + BAR_WIDTH / 2
    STATE.flappy_bar_y = math.random(0, PLAYFIELD_HEIGHT / 4 * 3) +
                             PLAYFIELD_HEIGHT / 8
end

function mod.render(STATE)
    image.blit(BACKGROUND_TEXTURE, 0, 0)
    screen.clip(DRAW_X_OFFSET, DRAW_Y_OFFSET, PLAYFIELD_WIDTH, PLAYFIELD_HEIGHT)
    image.blit(FLAPPY_TEXTURE, FLAPPY_X + DRAW_X_OFFSET,
               STATE.flappy_y + DRAW_Y_OFFSET)

    -- For some reason the centering causes a segfault, so math to position correctly is done here :(
    image.blit(PILLAR_TEXTURE_TOP,
               DRAW_X_OFFSET + STATE.flappy_bar_x - BAR_WIDTH / 2,
               DRAW_Y_OFFSET + STATE.flappy_bar_y - BAR_SPACING / 2 -
                   image.geth(PILLAR_TEXTURE_TOP))
    image.blit(PILLAR_TEXTURE_BOTTOM,
               DRAW_X_OFFSET + STATE.flappy_bar_x - BAR_WIDTH / 2,
               STATE.flappy_bar_y + BAR_SPACING / 2)

    screen.clip()
end

function mod.update(STATE)
    -- Buttons
    if buttons.cross then
        STATE.flappy_speed = -FLAPPY_SPEED_FLAPPING
        sound.play(FLAP_SOUND)
    else
        STATE.flappy_speed = STATE.flappy_speed + FLAPPY_SPEED_GRAVITY
    end

    -- Height
    STATE.flappy_y = STATE.flappy_y + STATE.flappy_speed

    -- Bars
    STATE.flappy_bar_x = STATE.flappy_bar_x - BAR_SPEED
    if STATE.flappy_bar_x < -BAR_WIDTH / 2 then
        STATE.flappy_bar_x = STATE.flappy_bar_x + PLAYFIELD_WIDTH + BAR_WIDTH
        STATE.flappy_bar_y = math.random(0, PLAYFIELD_HEIGHT / 4 * 3) +
                                 PLAYFIELD_HEIGHT / 8
        STATE.game_score = STATE.game_score + FLAPPY_LAP_SCORE
    end

    -- Collisions
    -- top rectangle
    if STATE.flappy_y - FLAPPY_SIZE / 2 > PLAYFIELD_HEIGHT then return true end
    if FLAPPY_X + FLAPPY_SIZE / 2 >= STATE.flappy_bar_x - BAR_WIDTH / 2 and
        FLAPPY_X - FLAPPY_SIZE / 2 <= STATE.flappy_bar_x + BAR_WIDTH / 2 then
        if STATE.flappy_y - FLAPPY_SIZE / 2 < STATE.flappy_bar_y - BAR_SPACING /
            2 or STATE.flappy_y + FLAPPY_SIZE / 2 > STATE.flappy_bar_y +
            BAR_SPACING / 2 then return true end
    end
end

function mod.stop_sounds() sound.stop(FLAP_SOUND) end

return mod
