local pong = require("pong")
local asteroid = require("asteroid")
local flappy = require("flappy")

local mod = {}

local GLITCHING_SOUND = sound.load("glitching.wav")

local GLITCH_TICKS_DURATION = 20 -- 1/3 seconds

local GAME_TICKS_DURATION = 60 * 7 -- 7 seconds

function mod.init_state(STATE)
    STATE.total_game_ticks = 0
    STATE.game_ticks = 0
    STATE.game_glitch_ticks = 0
    STATE.game_glitching = true
    STATE.game_score = 0
    STATE.current_game = 1
end

function mod.reset(STATE)
    pong.reset(STATE)
    pong.stop_sounds()
    asteroid.reset(STATE)
    asteroid.stop_sounds()
    flappy.reset(STATE)
    flappy.stop_sounds()
    mod.init_state(STATE)
end

local function game_by_name(name)
    if name == "pong" then return pong end
    if name == "asteroid" then return asteroid end
    if name == "flappy" then return flappy end
end

local function current_game(STATE)
    return game_by_name(STATE.lives[STATE.current_game])
end

function mod.render(STATE)
    if STATE.game_glitching then
        local glitch = math.random(10)
        if glitch == 1 then
            -- render wrong game
            game_by_name(STATE.lives[math.random(#STATE.lives)]).render(STATE)
            screen.print(0, 0, "Score: " .. STATE.game_score)
            screen.print(0, 20, "Lives: " .. #STATE.lives)
        elseif glitch == 2 then
            -- blank screen
        elseif glitch == 3 then
            -- half screen purple triangle
            if STATE.game_glitch_ticks % 2 == 1 then
                draw.filltriangle(0, 272, 0, 0, 480, 0, color.new(255, 0, 255))
            else
                draw.filltriangle(0, 272, 480, 0, 480, 272, color.new(255, 0, 255))
            end
        elseif glitch == 4 then
            -- turn off hardware LEDs
            for i=0, 3 do
                hw.led(i, 0)
            end
        elseif glitch == 5 then
            -- turn hardware LEDs back on
            for i=0, 3 do
                hw.led(i, 1)
            end
        elseif glitch == 6 then
            -- play glitching sound
            sound.play(GLITCHING_SOUND)
        else
            current_game(STATE).render(STATE)
            screen.print(0, 0, "Score: " .. STATE.game_score)
            screen.print(0, 20, "Lives: " .. #STATE.lives)
        end
    else
        current_game(STATE).render(STATE)
        screen.print(0, 0, "Score: " .. STATE.game_score)
        screen.print(0, 20, "Lives: " .. #STATE.lives)
    end
end

function mod.update(STATE)
    STATE.total_game_ticks = STATE.total_game_ticks + 1
    if STATE.game_glitching then
        STATE.game_glitch_ticks = STATE.game_glitch_ticks + 1
        if STATE.game_glitch_ticks >= GLITCH_TICKS_DURATION then
            STATE.game_glitch_ticks = 0
            STATE.game_glitching = false
            -- turn hardware LEDs back on
            for i=0, 3 do
                hw.led(i, 1)
            end
            -- stop glitching sound
            sound.stop(GLITCHING_SOUND)
        end
    else
        STATE.game_ticks = STATE.game_ticks + 1
        local lost_life = false
        lost_life = current_game(STATE).update(STATE)

        if lost_life then
            current_game(STATE).stop_sounds()
            table.remove(STATE.lives, STATE.current_game)
            if #STATE.lives == 0 then return "menu" end
            STATE.current_game = ((STATE.current_game - 1) % #STATE.lives) + 1
            STATE.game_ticks = 0
            STATE.game_glitching = true
        end

        if STATE.game_ticks >= GAME_TICKS_DURATION then
            STATE.game_ticks = 0
            STATE.game_glitching = true
            STATE.current_game = ((STATE.current_game) % #STATE.lives) + 1
        end
    end
end

return mod
