local mod = {}

local SELECT_SOUND = sound.load("menu_select.wav")
local ENTER_SOUND = sound.load("menu_enter.wav")

local BACKGROUND_TEXTURE = image.load("menu.png")

local DRAW_OFFSET_X = 130
local DRAW_OFFSET_Y = 100
local TEXT_COLOR = color.new(255, 255, 255)
local TEXT_SIZE = 0.75
local SHADOW_COLOR = color.new(150, 0, 150)

function mod.init_state(STATE)
    STATE.menu_index = 1
    STATE.menu_shown = mod.root_menu
end

function mod.reset(STATE)
    sound.stop(SELECT_SOUND)
    sound.stop(ENTER_SOUND)
end

function mod.start_normal(STATE)
    STATE.lives = {"pong", "asteroid", "flappy"}
    return "game"
end

function mod.show_training_menu(STATE)
    local function load_game(game)
        STATE.lives = {game}
        STATE.current_game = 1
        STATE.total_game_ticks = 0
        STATE.game_score = 0
        return "game"
    end

    local function load_pong(STATE) return load_game("pong", STATE) end

    local function load_asteroid(STATE) return load_game("asteroid", STATE) end

    local function load_flappy(STATE) return load_game("flappy", STATE) end

    local single_game_menu = {
        {label = "Pong", callback = load_pong},
        {label = "Asteroid", callback = load_asteroid},
        {label = "Flappy", callback = load_flappy},
        {label = "Cancel", callback = mod.init_state}
    }

    STATE.menu_shown = single_game_menu
    STATE.menu_index = 1
end

function mod.show_credits(STATE)
    local credits_menu = {
        {label = "Made by CaféHaine"}, {label = "(aka. psgarsenal)"},
        {label = "for Re:Start! 2022"}, {label = "Code under MIT license"},
        {label = "Graphics and sounds"}, {label = "under CC BY-NC 4.0"},
        {label = "Made using OneLua"}, {label = "OK", callback = mod.init_state}
    }
    STATE.menu_shown = credits_menu
    STATE.menu_index = #credits_menu
end

function mod.render(STATE)
    image.blit(BACKGROUND_TEXTURE, 0, 0)
    for index, item in ipairs(STATE.menu_shown) do
        screen.print(240, (index - 1) * 20 + DRAW_OFFSET_Y, item.label,
                     TEXT_SIZE, TEXT_COLOR, SHADOW_COLOR, __ACENTER)
        if index == STATE.menu_index then
            screen.print(DRAW_OFFSET_X, (index - 1) * 20 + DRAW_OFFSET_Y, "->",
                         TEXT_SIZE, TEXT_COLOR, SHADOW_COLOR)
            screen.print(480 - DRAW_OFFSET_X, (index - 1) * 20 + DRAW_OFFSET_Y,
                         "<-", TEXT_SIZE, TEXT_COLOR, SHADOW_COLOR, __ARIGHT)
        end
    end
end

function mod.update(STATE)
    if buttons.down then
        STATE.menu_index = STATE.menu_index + 1
        sound.play(SELECT_SOUND)
    end
    if buttons.up then
        STATE.menu_index = STATE.menu_index - 1
        sound.play(SELECT_SOUND)
    end
    STATE.menu_index = ((STATE.menu_index - 1) % #STATE.menu_shown) + 1

    if buttons.cross or buttons.circle then
        sound.play(ENTER_SOUND)
        if STATE.menu_shown[STATE.menu_index].callback ~= nil then
            return STATE.menu_shown[STATE.menu_index].callback(STATE)
        end
    end
end

mod.root_menu = {
    {label = "Normal mode", callback = mod.start_normal},
    {label = "Single game", callback = mod.show_training_menu},
    {label = "Credits", callback = mod.show_credits},
    {label = "Quit", callback = os.exit}
}

return mod
