local utils = require("utils")

local mod = {}

local WALL_HIT_SOUND = sound.load("pong_wall_hit.wav")
local PADDLE_HIT_SOUND = sound.load("pong_paddle_hit.wav")

local BACKGROUND_TEXTURE = image.load("pong.png")
local BALL_TEXTURE = image.load("pong_ball.png")
image.center(BALL_TEXTURE)
local PADDLE_TEXTURE = image.load("pong_paddle.png")
image.center(PADDLE_TEXTURE)

local GAME_WON_POINTS = 100

local BALL_SIZE = 10
local BALL_BASE_SPEED = 1.2
local BALL_LOG_DIVISOR = 30

local PLAYER_SPEED = 3
local ADVERSARY_SPEED_FACTOR = 0.87 -- 0.87 times the ball speed

local PADDLE_HEIGHT = 30
local PADDLE_WIDTH = 5

local PLAYFIELD_HEIGHT = 240
local PLAYFIELD_WIDTH = 320

local DRAW_X_OFFSET = 480 / 2 - PLAYFIELD_WIDTH / 2
local DRAW_Y_OFFSET = 272 / 2 - PLAYFIELD_HEIGHT / 2

function ball_speed(STATE)
    return BALL_BASE_SPEED + math.log(STATE.total_game_ticks) / 50
end

function mod.reset(STATE)
    STATE.pong_player_y = PLAYFIELD_HEIGHT / 2
    STATE.pong_ball_x = PLAYFIELD_WIDTH / 2
    STATE.pong_ball_y = PLAYFIELD_HEIGHT / 2
    STATE.pong_ball_direction_x = -1
    STATE.pong_ball_direction_y = 1
    STATE.pong_adversary_y = PLAYFIELD_HEIGHT / 2
end

function mod.render(STATE)
    image.blit(BACKGROUND_TEXTURE, 0, 0)

    screen.clip(DRAW_X_OFFSET, DRAW_Y_OFFSET, PLAYFIELD_WIDTH, PLAYFIELD_HEIGHT)
    image.blit(PADDLE_TEXTURE, DRAW_X_OFFSET + PADDLE_WIDTH / 2,
               STATE.pong_player_y + DRAW_Y_OFFSET)
    image.blit(PADDLE_TEXTURE,
               DRAW_X_OFFSET + PLAYFIELD_WIDTH - PADDLE_WIDTH / 2,
               STATE.pong_adversary_y + DRAW_Y_OFFSET)

    image.blit(BALL_TEXTURE, STATE.pong_ball_x + DRAW_X_OFFSET,
               STATE.pong_ball_y + DRAW_Y_OFFSET)

    screen.clip()
end

function mod.update(STATE)
    speed = ball_speed(STATE)

    -- Paddle movements
    if (buttons.held.up or buttons.analogy < -20) and STATE.pong_player_y > 0 +
        PADDLE_HEIGHT / 2 then
        STATE.pong_player_y = STATE.pong_player_y - PLAYER_SPEED
    end
    if (buttons.held.down or buttons.analogy > 20) and STATE.pong_player_y <
        PLAYFIELD_HEIGHT - PADDLE_HEIGHT / 2 then
        STATE.pong_player_y = STATE.pong_player_y + PLAYER_SPEED
    end

    -- Adversary movements
    if STATE.pong_ball_y < STATE.pong_adversary_y then
        STATE.pong_adversary_y = math.max(PADDLE_HEIGHT / 2,
                                          STATE.pong_adversary_y - speed *
                                              ADVERSARY_SPEED_FACTOR)
    end
    if STATE.pong_ball_y > STATE.pong_adversary_y then
        STATE.pong_adversary_y = math.min(
                                     PLAYFIELD_HEIGHT - PADDLE_HEIGHT / 2 - 1,
                                     STATE.pong_adversary_y + speed *
                                         ADVERSARY_SPEED_FACTOR)
    end

    -- Ball movements (Y)
    STATE.pong_ball_y = STATE.pong_ball_y + speed * STATE.pong_ball_direction_y
    if STATE.pong_ball_y < BALL_SIZE / 2 then
        local overshoot = STATE.pong_ball_y - BALL_SIZE / 2
        STATE.pong_ball_y = overshoot + BALL_SIZE / 2
        STATE.pong_ball_direction_y = 1
        sound.play(WALL_HIT_SOUND)
    end
    if STATE.pong_ball_y >= PLAYFIELD_HEIGHT - BALL_SIZE / 2 then
        local overshoot = PLAYFIELD_HEIGHT - BALL_SIZE / 2 - STATE.pong_ball_y
        STATE.pong_ball_y = PLAYFIELD_HEIGHT - BALL_SIZE / 2 - overshoot
        STATE.pong_ball_direction_y = -1
        sound.play(WALL_HIT_SOUND)
    end

    -- Ball movements (X)
    STATE.pong_ball_x = STATE.pong_ball_x + speed * STATE.pong_ball_direction_x
    if STATE.pong_ball_x <= PADDLE_WIDTH and STATE.pong_ball_x >= 0 and
        STATE.pong_ball_y >= STATE.pong_player_y - PADDLE_HEIGHT / 2 and
        STATE.pong_ball_y < STATE.pong_player_y + PADDLE_HEIGHT / 2 then
        local overshoot = -(STATE.pong_ball_x - BALL_SIZE / 2 - PADDLE_WIDTH)
        STATE.pong_ball_x = PADDLE_WIDTH + overshoot
        STATE.pong_ball_direction_x = 1
        if STATE.pong_ball_y > STATE.pong_player_y then
            STATE.pong_ball_direction_y = 1
        else
            STATE.pong_ball_direction_y = -1
        end
        sound.play(PADDLE_HIT_SOUND)
    end
    if STATE.pong_ball_x >= PLAYFIELD_WIDTH - 1 - PADDLE_WIDTH and
        STATE.pong_ball_x < PLAYFIELD_WIDTH and STATE.pong_ball_y >=
        STATE.pong_adversary_y - PADDLE_HEIGHT / 2 and STATE.pong_ball_y <
        STATE.pong_adversary_y + PADDLE_HEIGHT / 2 then
        local overshoot = PLAYFIELD_WIDTH - BALL_SIZE / 2 - STATE.pong_ball_x
        STATE.pong_ball_x = PLAYFIELD_WIDTH - PADDLE_WIDTH - 1 - overshoot
        STATE.pong_ball_direction_x = -1
        if STATE.pong_ball_y > STATE.pong_adversary_y then
            STATE.pong_ball_direction_y = 1
        else
            STATE.pong_ball_direction_y = -1
        end
        sound.play(PADDLE_HIT_SOUND)
    end

    -- Count points
    if STATE.pong_ball_x >= PLAYFIELD_WIDTH then -- game won
        STATE.game_score = STATE.game_score + GAME_WON_POINTS
        STATE.pong_ball_x = PLAYFIELD_WIDTH / 2
        STATE.pong_ball_y = PLAYFIELD_HEIGHT / 2
    end
    if STATE.pong_ball_x < 0 then -- life lost!
        return true
    end
end

function mod.stop_sounds()
    sound.stop(WALL_HIT_SOUND)
    sound.stop(PADDLE_HIT_SOUND)
end

return mod
